# The global hub for all the stats.
extends CanvasLayer

@onready var control = $Stats
@onready var categories = $Stats/Categories

# The key and string is shown like this 'key: value'
func stat_set(stat: Dictionary):
	# Check if label exists
	var found_key = false
	for child in categories.get_children():
		if child.name == stat["key"]:
			set_text(child, stat)
			found_key = true
	if not found_key:
		var new_label = RichTextLabel.new()
		new_label.name = stat["key"]
		categories.add_child(new_label)
		set_text(new_label, stat)

func set_text(node, stat):
	var key = ""
	var body = ""
	var end = ""
	# Make sure key exists
	if stat.has("key"):
		key = stat["key"]
	else:
		printerr("dictionary must have key")
	# Optional body
	if stat.has("value"):
		body = stat["value"]
	# Immediate indicators
	if stat.has("indicate"):
		end = "[color=orange]*[/color] "
	node.bbcode_enabled = true
	node.fit_content = true
	node.text = key + ": " + body + " " + end
	if stat.has("indicate"):
		await get_tree().create_timer(0.5).timeout
		node.text = key + ": " + body

# Utility function for your leasure
func maybe_put_stat(stat_key, value: String):
	if stat_key != null:
		stat_key["value"] = value
		stat_set(stat_key)

func _process(_delta):
	if Input.is_action_just_pressed("show_stats"):
		control.visible = !control.visible
	# Add fps stat
	stat_set({ "key": "fps", "value": str(Engine.get_frames_per_second()) })
