# An example of a script that could be on an enemy/player (in this case enemy).
# Controls state changes and state input such as direction, could use
# area detectors then change the state for example.
extends CharacterBody2D

func current_state():
	return stm.state

@onready var stm = $StateMachine
@onready var air = $StateMachine/Air
@onready var run = $StateMachine/Run
@onready var dash = $StateMachine/Dash

@onready var hitbox_up = $Hitboxes/Up
@onready var hitbox_down = $Hitboxes/Down
@onready var hitbox_side = $Hitboxes/Side

@export var buffer_jump := 0.0
var buffer_jump_tmp := 0.0

@export var coyote_jump := 0.0
var coyote_jump_tmp := coyote_jump

@export var double_jump_delay := 0.0
var double_jump_delay_tmp := 0.0
var double_jump_delay_initiated := false

@export var dash_time: float
var dash_time_tmp := dash_time

@export var hit_time: float
var hit_time_tmp: float
var hit_active := false

@export var pogo_jump: float

var double_jump_used := false
var dash_used := false
# Not an animation variable
var looking_right := true

func _ready():
	stm.state_debug_stat = { "key": "player state" }
	air.jump_debug_stat = { "key": "player jump curve" }
	air.air_run_debug_stat = { "key": "player run curve" }
	run.run_debug_stat = { "key": "player run curve" }

func update_looking_right():
	var dir = get_dir()
	if dir.x < 0:
		looking_right = false
	if dir.x > 0:
		looking_right = true

func _physics_process(delta):
	update_looking_right()
	if Input.is_action_just_pressed("jump"):
		buffer_jump_tmp = buffer_jump
	else:
		buffer_jump_tmp -= delta
	# Start falling
	if current_state() in [run] && not is_on_floor():
		if coyote_jump_tmp <= 0:
			# Fall
			coyote_jump_tmp = coyote_jump
			stm.transition(air)
		else:
			coyote_jump_tmp -= delta
			if Input.is_action_just_pressed("jump"):
				do_jump()
				StatHub.stat_set({ "key": "player did coyote jump", "indicate": true })
				return
	else:
		coyote_jump_tmp = coyote_jump
	# Land
	if current_state() in [air] && is_on_floor():
		double_jump_used = false
		dash_used = false
		if buffer_jump_tmp > 0:
			buffer_jump_tmp = 0
			double_jump_delay_initiated = false
			stm.transition(air, { "jump_stats" : run.jump() })
			StatHub.stat_set({ "key": "player jumped buffered jump", "indicate": true })
		else:
			stm.transition(run)
	# Jump
	if current_state() in [run] && Input.is_action_just_pressed("jump"):
		do_jump()
		return
	# Double Jump
	if current_state() in [air]:
		if double_jump_delay_initiated:
			if double_jump_delay_tmp <= 0 && !double_jump_used:
				double_jump_delay_initiated = false
				double_jump_used = true
				do_jump()
				StatHub.stat_set({ "key": "player did double jump", "indicate": true })
			else:
				double_jump_delay_tmp -= delta
		elif Input.is_action_just_pressed("jump"):
			double_jump_delay_initiated = true
			double_jump_delay_tmp = double_jump_delay
	else:
		double_jump_delay_initiated = false
	# Continue jumping
	if current_state() in [air]:
		current_state().continue_jump(Input.is_action_pressed("jump"))
	# Walk left/right
	if current_state() in [run, air]:
		current_state().direction = get_dir().x
	# Do dash
	if current_state() in [run, air] && Input.is_action_just_pressed("dash") && !dash_used:
		dash_used = true
		dash_time_tmp = dash_time
		var dir := get_dir()
		if dir == Vector2.ZERO:
			if looking_right: dir.x = 1
			else:             dir.x = -1
		stm.transition(dash)
		current_state().direction = dir
	# Done with dash
	if current_state() in [dash]:
		if dash_time_tmp <= 0:
			stm.transition(air)  # TODO: maybe ground?
		else:
			dash_time_tmp -= delta
	# Hitboxes
	if current_state() in [run, air]:
		if hit_active:
			if hit_time_tmp <= 0:
				hit_active = false
				disable_hitboxes()
			else:
				hit_time_tmp -= delta
		else:
			if looking_right: hitbox_side.get_parent().scale.x = 1
			else:             hitbox_side.get_parent().scale.x = -1
			if Input.is_action_just_pressed("attack"):
				hit_active = true
				hit_time_tmp = hit_time
				var dir = get_dir()
				if dir.y < 0:
					hitbox_up.set_active(true)
				elif dir.y > 0:
					hitbox_down.set_active(true)
				else:
					hitbox_side.set_active(true)
	else:
		disable_hitboxes()
	
	current_state().update(delta)
	StatHub.stat_set({ "key": "player velocity", "value": str(velocity) })

func disable_hitboxes():
	for hitbox in [hitbox_up, hitbox_down, hitbox_side]:
		hitbox.set_active(false)

func do_jump():
	stm.transition(air, { "jump_stats" : current_state().jump() })

func get_dir() -> Vector2:
	var analog := Vector2.ZERO
	if Input.is_action_pressed("walk_left_analog"):
		analog.x += -1
	if Input.is_action_pressed("walk_right_analog"):
		analog.x += 1
	if Input.is_action_pressed("walk_up_analog"):
		analog.y += -1
	if Input.is_action_pressed("walk_down_analog"):
		analog.y += 1
	var digital = Vector2(Input.get_axis("walk_left", "walk_right"), Input.get_axis("walk_up", "walk_down"))
	if analog != Vector2.ZERO: return analog
	else:                      return digital

func _on_down_hit(_data):
	dash_used = false
	double_jump_used = false
	velocity.y = - pogo_jump

func _on_hurt_box_hurt(_data):
	print("Player: ouch!")
